@@ -9,16 +9,16 @@ use strict;
 use Lemonldap::NG::Portal::Simple;
 use AuthCAS;
 
-our $VERSION = '0.04';
+our $VERSION = '0.1';
 
-## @method int authInit()
+## @apmethod int authInit()
 # Does nothing.
 # @return Lemonldap::NG::Portal constant
 sub authInit {
     PE_OK;
 }
 
-## @method int extractFormInfo()
+## @apmethod int extractFormInfo()
 # Read username return by CAS authentication system.
 # If user isn't authenticated, redirect it to CAS portal.
 # @return Lemonldap::NG::Portal constant
@@ -45,7 +45,19 @@ sub extractFormInfo {
     PE_OK;
 }
 
-## @method int authenticate()
+## @apmethod int setAuthSessionInfo()
+# Does nothing here.
+# @return Lemonldap::NG::Portal constant
+sub setAuthSessionInfo {
+    my $self = shift;
+
+    # Store user submitted login for basic rules
+    $self->{sessionInfo}->{'_user'} = $self->{'user'};
+
+    PE_OK;
+}
+
+## @apmethod int authenticate()
 # Does nothing.
 # @return Lemonldap::NG::Portal constant
 sub authenticate {
